@@ -246,10 +246,30 @@ fn scan_for_transaction(
                 let length = (scratch_buffer[1] as usize) << 16
                     | (scratch_buffer[2] as usize) << 8
                     | scratch_buffer[3] as usize;
-                if scratch_buffer.len() < length {
-                    scratch_buffer.resize(length, 0);
+                scratch_buffer.resize(length, 0);
+                let mut initial_page = true;
+                let mut bytes_to_read = length;
+                let mut offset = 0;
+                while bytes_to_read > 0 {
+                    let page_header_length = if initial_page {
+                        // The initial page has 4 bytes at the start, which we've already read.
+                        initial_page = false;
+                        4
+                    } else {
+                        // Subsequent pages have a 0 byte at the start of the
+                        // page, denoting that it's not a valid page header. We
+                        // need to skip that byte, so that the read call reads
+                        // the stored data, not the header byte.
+                        log.seek(SeekFrom::Current(1))?;
+                        1
+                    };
+
+                    let page_length = (PAGE_SIZE - page_header_length).min(length - offset);
+                    log.read_exact(&mut scratch_buffer[offset..offset + page_length])?;
+                    offset += page_length;
+                    bytes_to_read -= page_length;
                 }
-                log.read_exact(scratch_buffer)?;
+
                 let payload = &scratch_buffer[0..length];
                 let decrypted = match &vault {
                     Some(vault) => Cow::Owned(vault.decrypt(payload)?),
@@ -517,7 +537,7 @@ impl<'a> LogEntry<'a> {
     }
 
     pub(crate) fn serialize(&self) -> Result<Vec<u8>, Error> {
-        let mut buffer = Vec::new();
+        let mut buffer = Vec::with_capacity(8 + self.data.as_ref().map_or(0, |data| data.len()));
         // Transaction ID
         buffer.write_u64::<BigEndian>(self.id)?;
         if let Some(data) = &self.data {
@@ -601,12 +621,20 @@ fn guess_page(
         let local_avg_per_page = (upper_id - lower_id) as f64 / (total_pages - current_page) as f64;
         let delta_estimated_pages = (delta_from_current as f64 * local_avg_per_page).floor() as u64;
         let guess = lower_location + delta_estimated_pages.max(1) * PAGE_SIZE as u64;
-        // If our estimate is that the location is beyond or equal to the upper, we'll guess the page before it.
-        Some(if guess >= upper_location {
-            upper_location - PAGE_SIZE as u64
+        // If our estimate is that the location is beyond or equal to the upper,
+        // we'll guess the page before it.
+        if guess >= upper_location {
+            let capped_guess = upper_location - PAGE_SIZE as u64;
+            // If the page before the upper is the lower, we can't find the
+            // transaction in question.
+            if capped_guess > lower_location {
+                Some(capped_guess)
+            } else {
+                None
+            }
         } else {
-            guess
-        })
+            Some(guess)
+        }
     } else if upper_id > looking_for {
         // Go backwards from upper
         let avg_per_page = upper_id as f64 / total_pages as f64;
@@ -623,6 +651,8 @@ fn guess_page(
 #[allow(clippy::semicolon_if_nothing_returned, clippy::future_not_send)]
 mod tests {
 
+    use std::collections::HashSet;
+
     use nanorand::{Pcg64, Rng};
     use tempfile::tempdir;
 
@@ -684,6 +714,11 @@ mod tests {
             directory.join("_transactions")
         };
 
+        let mut rng = Pcg64::new_seed(1);
+        let data = (0..PAGE_SIZE * 10)
+            .map(|_| rng.generate())
+            .collect::<Vec<u8>>();
+
         for id in 1..=1_000 {
             let state = State::from_path(&log_path);
             TransactionLog::<Manager::File>::initialize_state(&state, &context).unwrap();
@@ -698,16 +733,18 @@ mod tests {
             if id % 2 == 0 {
                 // Larger than PAGE_SIZE
                 if id % 3 == 0 {
-                    tx.set_data(vec![42; PAGE_SIZE * (id as usize % 10).min(3)])
+                    tx.set_data(data[0..PAGE_SIZE * (id as usize % 10).max(3)].to_vec())
                         .unwrap();
                 } else {
-                    tx.set_data(vec![42; PAGE_SIZE * (id as usize % 10).min(2)])
+                    tx.set_data(data[0..PAGE_SIZE * (id as usize % 10).max(2)].to_vec())
                         .unwrap();
                 }
             } else {
-                tx.set_data(vec![41; id as usize]).unwrap();
+                tx.set_data(data[0..id as usize].to_vec()).unwrap();
             }
 
+            assert!(tx.data.as_ref().unwrap().len() > 0);
+
             transactions.push(vec![tx.transaction]).unwrap();
             transactions.close().unwrap();
         }
@@ -736,7 +773,13 @@ mod tests {
         for id in 1..=1_000 {
             let transaction = transactions.get(id).unwrap();
             match transaction {
-                Some(transaction) => assert_eq!(transaction.id, id),
+                Some(transaction) => {
+                    assert_eq!(transaction.id, id);
+                    assert_eq!(
+                        &data[..transaction.data().unwrap().len()],
+                        transaction.data().unwrap().as_slice()
+                    );
+                }
                 None => {
                     unreachable!("failed to fetch transaction {}", id)
                 }
@@ -779,7 +822,7 @@ mod tests {
         TransactionLog::<StdFile>::initialize_state(&state, &context).unwrap();
         let mut transactions = TransactionLog::<StdFile>::open(&log_path, state, context).unwrap();
 
-        let mut valid_ids = Vec::new();
+        let mut valid_ids = HashSet::new();
         for id in 1..=10_000 {
             assert_eq!(transactions.current_transaction_id(), id);
             let tx = transactions.new_transaction([&b"hello"[..]]);
@@ -787,17 +830,17 @@ mod tests {
                 // skip a few ids.
                 continue;
             }
-            valid_ids.push(tx.id);
+            valid_ids.insert(tx.id);
 
             transactions.push(vec![tx.transaction]).unwrap();
         }
 
-        for id in valid_ids {
+        for id in 1..=10_000 {
             let transaction = transactions.get(id).unwrap();
             match transaction {
                 Some(transaction) => assert_eq!(transaction.id, id),
                 None => {
-                    unreachable!("failed to fetch transaction {}", id)
+                    assert!(!valid_ids.contains(&id));
                 }
             }
         }
